use diesel::prelude::*;
use diesel::result::Error;
use serde::{Deserialize, Serialize};

use crate::helpers::db;

use super::artists::Artists;
use super::tracks::Tracks;

use crate::schema::{artists, artists_tracks, tracks};

#[derive(Identifiable, Selectable, Queryable, Associations, Debug)]
#[diesel(belongs_to(Artists, foreign_key = artist_id))]
#[diesel(belongs_to(Tracks, foreign_key = track_id))]
#[diesel(table_name = artists_tracks)]
#[diesel(primary_key(artist_id, track_id))]
pub struct ArtistTracks {
    pub artist_id: String,
    pub track_id: String,
}

impl ArtistTracks {
    pub fn get_tracks(artist: &Artists) -> Result<Vec<Tracks>, Error> {
        let conn = &mut db::connection()?;
        let track_ids = ArtistTracks::belonging_to(artist).select(artists_tracks::track_id);

        let tracks: Vec<Tracks> = tracks::table
            .filter(tracks::id.eq_any(track_ids))
            .load::<Tracks>(conn)?;

        Ok(tracks)
    }

    pub fn get_artists(track: &Tracks) -> Result<Vec<TrackArtist>, Error> {
        let conn = &mut db::connection()?;
        let artist_ids = ArtistTracks::belonging_to(track).select(artists_tracks::artist_id);

        let artists = artists::table
            .select((artists::id, artists::name))
            .filter(artists::id.eq_any(artist_ids))
            .load::<TrackArtist>(conn)?;

        Ok(artists)
    }
}

#[derive(Debug, Deserialize, Serialize, Queryable)]
#[diesel(table_name = artists)]
pub struct TrackArtist {
    pub id: String,
    pub name: String,
}
